@@ -1,4 +1,4 @@
-# $Id: /mirror/gungho/lib/Gungho/Component/RobotRules.pm 4564 2007-10-29T16:25:42.369911Z lestrrat  $
+# $Id: /mirror/gungho/lib/Gungho/Component/RobotRules.pm 7191 2007-05-15T02:45:51.609363Z lestrrat  $
 #
 # Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
 
@@ -29,13 +29,18 @@ sub send_request
         $c->allowed($request)
     ;
     if ($allowed == -2) {
-        $c->log->debug("Fetch for /robots.txt already scheduled for " . $request->original_uri->host_port);
+        $c->log->debug("Fetch for /robots.txt already scheduled for " . $request->uri->host_port)
+            if $c->log->is_debug;
+        Gungho::Exception::SendRequest::Handled->throw;
     } elsif ($allowed == -1) {
-        $c->log->debug("No robot rules found for " . $request->original_uri->host_port . ", going to fetch one");
+        $c->log->debug("No robot rules found for " . $request->uri->host_port . ", going to fetch one")
+            if $c->log->is_debug;
+        Gungho::Exception::SendRequest::Handled->throw;
     } elsif ($allowed) {
-        $c->next::method($request);
+        $c->maybe::next::method($request);
     } else {
         $c->log->debug($request->uri . " is disallowed by robot rules");
+        Gungho::Exception::SendRequest::Handled->throw;
     }
 }
 
@@ -43,7 +48,7 @@ sub allowed
 {
     my ($c, $request) = @_;
 
-    my $rule = $c->robot_rules_storage->get_rule( $c, $request );
+    my $rule = $c->robot_rules_storage->get_rule( $request );
     if (! $rule) {
         if ($c->push_pending_robots_txt($request) == 0) {
             return -2;
@@ -51,8 +56,6 @@ sub allowed
 
         my $uri = $request->original_uri;
         $uri->path('/robots.txt');
-        $uri->query(undef);
-        $uri->fragment(undef);
         my $req = Gungho::Request->new(GET => $uri);
         $req->notes('auto_robot_rules' => 1);
         $c->provider->pushback_request( $c, $req );
@@ -68,26 +71,42 @@ sub handle_response
     my ($request, $response) = @_;
 
     if ($request->uri->path eq '/robots.txt' && $request->notes('auto_robot_rules')) {
-        $c->log->debug("Handling robots.txt response for " . $request->uri);
-        $c->parse_robot_rules($request, $response);
+        if ($response->is_success) {
+            $c->parse_robot_rules($request, $response);
+        }
         $c->dispatch_pending_robots_txt($request);
-        return;
+        Gungho::Exception::HandleResponse::Handle->throw;
     }
 
-    $c->next::method(@_);
+    $c->maybe::next::method(@_);
 }
 
 sub push_pending_robots_txt
 {
     my ($c, $request) = @_;
-    return $c->robot_rules_storage->push_pending_robots_txt( $c, $request );
+
+    my $uri = $request->original_uri;
+    my $h = $c->pending_robots_txt->{ $uri->host_port };
+    if (! $h) {
+        $h = {};
+        $c->pending_robots_txt->{ $uri->host_port } = $h;
+    }
+
+    if(! exists $h->{ $request->id }) {
+        $c->log->debug("Pushing request " . $request->id . " to pending list (robot rules)...")
+            if $c->log->is_debug;
+        $h->{ $request->id } = $request ;
+        return 1;
+    }
+    return 0;
 }
 
 sub dispatch_pending_robots_txt
 {
     my ($c, $request) = @_;
 
-    my $pending = $c->robot_rules_storage->get_pending_robots_txt($c, $request);
+    my $uri = $request->original_uri;
+    my $pending = delete $c->pending_robots_txt->{ $uri->host_port };
     if ($pending && ref $pending eq 'HASH') {
         $c->provider->pushback_request( $c, $_ ) for values %$pending;
     }
@@ -102,8 +121,8 @@ sub setup_robot_rules_storage
     my $pkg = $config->{module} || 'RobotRules::Storage::DB_File';
     my $pkg_config = $config->{config} || {};
     $pkg = $c->load_gungho_module($pkg, 'Component');
-    my $storage = $pkg->new(%$config);
-    $storage->setup($c);
+    my $storage = $pkg->new($config);
+    $storage->setup();
     $c->robot_rules_storage( $storage );
 }
 
@@ -124,20 +143,19 @@ sub parse_robot_rules
 {
     my ($c, $request, $response) = @_;
 
-    my $h = ($request && $response && $response->is_success && $response->content) ?
+    my $h = $response->content ?
         $c->robot_rules_parser->parse($request->original_uri, $response->content) :
         {}
     ;
-    $c->log->debug("Parse robot rules " . $request->uri . ": " . keys(%$h) . " rules");
     my $rule = Gungho::Component::RobotRules::Rule->new($h);
-    $c->robot_rules_storage->put_rule($c, $request, $rule);
+    $c->robot_rules_storage->put_rule($request, $rule);
 }
 
 1;
 
 =head1 NAME
 
-Gungho::Component::RobotRules - Respect robots.txt
+Gungho::Compoent::RobotRules - Respect robots.txt
 
 =head1 SYNOPSIS
 
